/**
 **   vect_to_tori - convert AVS vect file to a series of AVS mesh files
 ** 
 **  CREATED:  2002.02.03  ABS  copied from vect_to_meshes
 **
 **/

#include        "vect_to_tori.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
void help(char** argv) {
}
void usage(char** argv) {
}
void defaultParms();
void parseArgs(int argc, char** argv);
void xyz_to_rphitheta(float xIn, float yIn, float zIn, float* p_r, float* p_phi, float* p_theta);


void main(int argc, char** argv)

{

    /* declare local variables */

    char filename[255];
    float x1, y1, z1, x2, y2, z2;
    float phi, theta;
    float x, y, z;
    float x_new, y_new, z_new;
    float x_temp, y_temp, z_temp;
    float a1, a2;
    int n, i, j, k;
    FILE *meshfile;
    FILE *datafile;
    float vertices[3][2048];
    float length;



	/* initialize global variables (including parameters) */

    Pi = 4.0*atan(1.0);
    defaultParms();
    parseArgs(argc, argv);

    /* read & write */

    datafile = fopen("data.txt", "r");
	int NumVects = 15;
    fscanf(stdin, "%d", &NumVects);
    for (n = 0; n < NumVects; n++) {
        fscanf(datafile, "%f", &Radius);
        Radius = sqrt(Radius)*0.1;
        if (Debug != FALSE) {
            fprintf(stderr, "%s: DEBUG: Radius = %f\n", argv[0], Radius);
            }
        fscanf(stdin, "%f %f %f %f %f %f", &x1, &y1, &z1, &x2, &y2, &z2);
        x = x2 - x1;
        y = y2 - y1;
        z = z2 - z1;
        xyz_to_rphitheta(x, y, z, &length, &phi, &theta);
        theta = theta*-1.0; /* I like left handed... */
        sprintf(filename, "t%d.mesh", n);
        if (Debug != FALSE) {
            fprintf(stderr, "%s: DEBUG: InnerRadius = %f\n", argv[0], InnerRadius);
            fprintf(stderr, "%s: DEBUG: filename = %s\n", argv[0], filename);
            fprintf(stderr, "%s: DEBUG: length = %f\n", argv[0], length);
            fprintf(stderr, "%s: DEBUG: phi = %f\n", argv[0], phi);
            fprintf(stderr, "%s: DEBUG: theta = %f\n", argv[0], theta);
            fprintf(stderr, "%s: DEBUG: x1 = %f\n", argv[0], x1);
            fprintf(stderr, "%s: DEBUG: x = %f\n", argv[0], x);
            }
        meshfile = fopen(filename, "w");
        //fprintf(meshfile, "vertex\n");
#ifdef RINGS
        fprintf(meshfile, "%d %d\n", 2, Facets + 1);
        for (i = 0; i < Facets + 1; i++) {
            vertices[0][i] = 0.0;
            vertices[1][i] = Radius*-sin(2.0*Pi*(float)i/(float)Facets);
            vertices[2][i] = Radius*cos(2.0*Pi*(float)i/(float)Facets);
            }
        j = Facets + 1;
        for (i = 0; i < Facets + 1; i++) {
            vertices[0][j] = 0.1;
            vertices[1][j] = Radius*-sin(2.0*Pi*(float)i/(float)Facets);
            vertices[2][j] = Radius*cos(2.0*Pi*(float)i/(float)Facets);
            j++;
            }
        for (i = 0; i < 2*(Facets + 1); i++) {
#endif RINGS
#ifndef RINGS
        //fprintf(meshfile, "%d %d\n", Facets + 1, Facets + 1);
        k = 0;
        for (i = 0; i < Facets + 1; i++) {
            a1 = 2.0*Pi*(float)i/(float)Facets;
            for (j = 0; j < Facets + 1; j++) {
                a2 = -2.0*Pi*(float)j/(float)Facets;
                x_temp =          InnerRadius*cos(a2);
                y_temp = 0.0;
                z_temp = Radius + InnerRadius*sin(a2);
                vertices[0][k] =  x_temp;
                vertices[1][k] =  y_temp*cos(a1) + z_temp*sin(a1);
                vertices[2][k] = -y_temp*sin(a1) + z_temp*cos(a1);
                k++;
                }
            }
        for (i = 0; i < (Facets + 1)*(Facets + 1); i++) {
#endif ifndef RINGS

            /* rotate phi about z axis */

            x_new = cos(phi)*vertices[0][i] - sin(phi)*vertices[1][i];;
            y_new = sin(phi)*vertices[0][i] + cos(phi)*vertices[1][i];;
            z_new = vertices[2][i];
            vertices[0][i] = x_new;
            vertices[1][i] = y_new;
            vertices[2][i] = z_new;

            /* rotate theta about y axis */

            x_new = cos(theta)*vertices[0][i] + sin(theta)*vertices[2][i];;
            y_new = vertices[1][i];
            z_new = -sin(theta)*vertices[0][i] + cos(theta)*vertices[2][i];;
            vertices[0][i] = x_new;
            vertices[1][i] = y_new;
            vertices[2][i] = z_new;

            /* translate to head position */

            vertices[0][i] = vertices[0][i] + x2;
            vertices[1][i] = vertices[1][i] + y2;
            vertices[2][i] = vertices[2][i] + z2;
            }
#ifdef RINGS
        for (i = 0; i < 2*(Facets + 1); i++) {
#endif RINGS
#ifndef RINGS
        for (i = 0; i < (Facets + 1)*(Facets + 1); i++) {
#endif ifndef RINGS
			if ((i % (Facets+1)) == 0 )
            fprintf(meshfile, "%f %f %f\n",
                vertices[0][i], vertices[1][i], vertices[2][i]);
            }
        fclose(meshfile);
        }

    } /* end main */

/*
 *  defaultParms - initialize global variables
 */
void defaultParms()
{
    Debug = FALSE;

    Facets = 16;
    Radius = 0.1;

    } /* end defaultParms() */

/*
 *  parseArgs - parse arguments into global variables
 */
void parseArgs(int argc, char** argv)
{
    int i;

    for (i = 0; i < argc; i++) {
        if (argv[i][0] == '-') {
            switch (argv[i][1]) {
                case 'D' : Debug = TRUE;
                           break;
                case 'f' : Facets = atoi(&argv[++i][0]);
                           break;
                case 'h' : help(argv);
                           exit(0);
                           break;
                case 'i' : InnerRadius = atof(&argv[++i][0]);
                           break;
                case 'u' : usage(argv);
                           exit(0);
                           break;
                default:   fprintf(stderr, "%s: ERROR: illegal flag: %s\n",
                               argv[0], argv[i]);
                           usage(argv);
                           exit(0);
                           break;
                } /* end switch */
            } /* end if */
        } /* end for(i) */
    } /* end parseArgs() */

void xyz_to_rphitheta(float xIn, float yIn, float zIn, float* p_r, float* p_phi, float* p_theta)
{
    if (Debug != FALSE) {
    double atan();
    double asin();

        fprintf(stderr, "xyz_to_rphitheta: DEBUG: xIn = %f\n", xIn);
        fprintf(stderr, "xyz_to_rphitheta: DEBUG: yIn = %f\n", yIn);
        fprintf(stderr, "xyz_to_rphitheta: DEBUG: zIn = %f\n", zIn);
        }
    *p_r = sqrt(xIn*xIn + yIn*yIn + zIn*zIn);
    if (xIn == 0.0) {
        if (zIn >= 0.0) {
            *p_theta = Pi/2.0;
            }
        else {
            *p_theta = -Pi/2.0;
            }
        }
    else if (xIn < 0.0) {
        *p_theta = Pi + atan(zIn/xIn);
        }
    else {
        *p_theta = atan(zIn/xIn);
        }
    if (*p_r == 0.0) {
        *p_phi = 0.0;
        }
    else {
        *p_phi = asin(yIn/ *p_r);
        }
    } /* end xyz_to_rphitheta() */

